from .provider.doodstream import doodstream_get_direct_link
from .provider.streamtape import streamtape_get_direct_link
from .provider.vidoza import vidoza_get_direct_link
from .provider.voe import voe_get_direct_link
from .provider.vidmoly import vidmoly_get_direct_link
from .provider.speedfiles import speedfiles_get_direct_link

from .nhentai.nhentai import nhentai
from .streamkiste.streamkiste import streamkiste
from .jav.jav import jav
from .hanime.hanime import hanime
